@@ -16,7 +16,7 @@
 mxRun <- function(model, ..., intervals=NULL, silent = FALSE, 
 		suppressWarnings = FALSE, unsafe = FALSE,
 		checkpoint = FALSE, useSocket = FALSE, onlyFrontend = FALSE, 
-		useOptimizer = TRUE){
+		useOptimizer = TRUE, beginMessage=!silent){
 
 	warnModelCreatedByOldVersion(model)
 
@@ -37,12 +37,13 @@ mxRun <- function(model, ..., intervals=NULL, silent = FALSE,
 	}
 	runHelper(model, frontendStart, intervals,
 		silent, suppressWarnings, unsafe,
-		checkpoint, useSocket, onlyFrontend, useOptimizer)
+		checkpoint, useSocket, onlyFrontend, useOptimizer, beginMessage)
 }
 
 runHelper <- function(model, frontendStart, 
 		intervals, silent, suppressWarnings, 
-		unsafe, checkpoint, useSocket, onlyFrontend, useOptimizer, parentData = NULL) {
+		unsafe, checkpoint, useSocket, onlyFrontend, useOptimizer,
+		beginMessage, parentData = NULL) {
 
 	Rcpp::Module  # ensure Rcpp is loaded
 	model <- imxPreprocessModel(model)
@@ -59,7 +60,8 @@ runHelper <- function(model, frontendStart,
 		intervals = intervals, silent = silent, 
 		suppressWarnings = suppressWarnings, unsafe = unsafe,
 		checkpoint = checkpoint, useSocket = useSocket,
-		onlyFrontend = onlyFrontend, useOptimizer = useOptimizer, parentData = model@data)
+		onlyFrontend = onlyFrontend, useOptimizer = useOptimizer,
+		beginMessage=beginMessage, parentData = model@data)
 		indepTimeStop <- Sys.time()
 		indepElapsed <- indepTimeStop - indepTimeStart
 		return(processHollowModel(model, independents, 
@@ -74,7 +76,8 @@ runHelper <- function(model, frontendStart,
 		intervals = intervals, silent = silent, 
 		suppressWarnings = suppressWarnings, unsafe = unsafe,
 		checkpoint = checkpoint, useSocket = useSocket,
-		onlyFrontend = onlyFrontend, useOptimizer = useOptimizer)
+		onlyFrontend = onlyFrontend, beginMessage=beginMessage,
+		useOptimizer = useOptimizer)
 	indepTimeStop <- Sys.time()
 	indepElapsed <- indepTimeStop - indepTimeStart
 	if (modelIsHollow(model)) {
@@ -122,7 +125,7 @@ runHelper <- function(model, frontendStart,
 	defVars <- generateDefinitionList(flatModel, list())
 	model <- expectationFunctionAddEntities(model, flatModel, labelsData)
 	model <- preprocessDatasets(model, defVars, model@options) # DEPRECATED
-	flatModel@datasets <- collectDatasets(model)  # done in imxFlattenModel, but confusingly do it again
+	flatModel@datasets <- collectDatasets(model, namespace)  # done in imxFlattenModel, but confusingly do it again
 	labelsData <- imxGenerateLabels(model)
 
 	model <- fitFunctionAddEntities(model, flatModel, labelsData)
@@ -208,7 +211,7 @@ runHelper <- function(model, frontendStart,
 	
 	frontendStop <- Sys.time()
 	frontendElapsed <- (frontendStop - frontendStart) - indepElapsed
-	if(!silent) message("Running ", model@name, " with ", numParam, " parameter",
+	if(beginMessage) message("Running ", model@name, " with ", numParam, " parameter",
 			    ifelse(numParam==1, "", "s"))
 	if (onlyFrontend) return(model)
 
@@ -219,11 +222,13 @@ runHelper <- function(model, frontendStart,
 			silent || !interactive(), PACKAGE = "OpenMx")
 	backendStop <- Sys.time()
 	backendElapsed <- backendStop - frontendStop
-	model <- updateModelMatrices(model, flatModel, output$matrices)
-	model <- updateModelAlgebras(model, flatModel, output$algebras)
-	model <- updateModelExpectations(model, flatModel, output$expectations)
-	model <- updateModelExpectationDims(model, expectations)
-	model <- updateModelData(model, flatModel, output$data)
+	if (is.null(output$error)) {
+		model <- updateModelMatrices(model, flatModel, output$matrices)
+		model <- updateModelAlgebras(model, flatModel, output$algebras)
+		model <- updateModelExpectations(model, flatModel, output$expectations)
+		model <- updateModelExpectationDims(model, expectations)
+		model <- updateModelData(model, flatModel, output$data)
+	}
 	model@compute <-updateModelCompute(model, output$computes)
 	output[['computes']] <- NULL
 	if (!is.null(output[['bounds']])) {
@@ -350,6 +355,7 @@ mxBootstrap <- function(model, replications=200, ...,
     if (missing(data)) {
       data <- enumerateDatasets(model)
     }
+	    # wrap with mxComputeTryCatch ?
     plan <- mxComputeBootstrap(data, model@compute)
   } else {
     plan <- model$compute
@@ -372,8 +378,7 @@ omxGetBootstrapReplications <- function(model) {
   if (is.null(model$compute) || !is(model$compute, "MxComputeBootstrap")) {
 	  stop(paste("Compute plan", class(model$compute), "found in model",
 		     omxQuotes(model$name),
-		     "instead of MxComputeBootstrap. Have you run this model",
-		     "through mxBootstrap already?"))
+		     "instead of MxComputeBootstrap. You need to run this model through mxBootstrap first."))
   }
    assertModelFreshlyRun(model)
   cb <- model@compute
