@@ -133,12 +133,12 @@ static void buildLatentParamMap(omxFitFunction* oo, FitContext *fc)
 						fv->lbound = estate->grp.quad.MIN_VARIANCE;  // variance must be positive
 						Global->boundsUpdated = true;
 						if (fc->est[px] < fv->lbound) {
-							Rf_error("Starting value for variance %s is not positive", fv->name);
+							mxThrow("Starting value for variance %s is not positive", fv->name);
 						}
 					}
 				} else if (latentMap[cell] != px) {
 					// doesn't detect similar problems in multigroup constraints TODO
-					Rf_error("Covariance matrix must be constrained to preserve symmetry");
+					mxThrow("Covariance matrix must be constrained to preserve symmetry");
 				}
 				state->freeLatents = true;
 			}
@@ -205,14 +205,14 @@ static void buildItemParamMap(omxFitFunction* oo, FitContext *fc)
 			if (state->paramFlavor[px] == 0) {
 				state->paramFlavor[px] = flavor;
 			} else if (strcmp(state->paramFlavor[px], flavor) != 0) {
-				Rf_error("Cannot equate %s with %s[%d,%d]", fv->name,
+				mxThrow("Cannot equate %s with %s[%d,%d]", fv->name,
 					 itemParam->name(), loc->row, loc->col);
 			}
 			if (fv->lbound == NEG_INF && std::isfinite(lower)) {
 				fv->lbound = lower;
 				Global->boundsUpdated = true;
 				if (fc->est[px] < fv->lbound) {
-					Rf_error("Starting value %s %f less than lower bound %f",
+					mxThrow("Starting value %s %f less than lower bound %f",
 					      fv->name, fc->est[px], lower);
 				}
 			}
@@ -220,7 +220,7 @@ static void buildItemParamMap(omxFitFunction* oo, FitContext *fc)
 				fv->ubound = upper;
 				Global->boundsUpdated = true;
 				if (fc->est[px] > fv->ubound) {
-					Rf_error("Starting value %s %f greater than upper bound %f",
+					mxThrow("Starting value %s %f greater than upper bound %f",
 					      fv->name, fc->est[px], upper);
 				}
 			}
@@ -244,16 +244,16 @@ static void buildItemParamMap(omxFitFunction* oo, FitContext *fc)
 			const int outer_at1 = state->paramMap[cx * state->itemDerivPadSize + p1];
 			if (outer_at1 < 0) continue;
 			const int outer_hb1 = std::lower_bound(hb.vars.begin(), hb.vars.end(), outer_at1) - hb.vars.begin();
-			if (hb.vars[outer_hb1] != outer_at1) Rf_error("oops");
+			if (hb.vars[outer_hb1] != outer_at1) mxThrow("oops");
 
 			for (int p2=0; p2 <= p1; p2++) {
 				int at1 = outer_at1;
 				int hb1 = outer_hb1;
 				int at2 = state->paramMap[cx * state->itemDerivPadSize + p2];
 				if (at2 < 0) continue;
-				if (p1 == p2 && at1 != at2) Rf_error("oops");
+				if (p1 == p2 && at1 != at2) mxThrow("oops");
 				int hb2 = std::lower_bound(hb.vars.begin(), hb.vars.end(), at2) - hb.vars.begin();
-				if (hb.vars[hb2] != at2) Rf_error("oops");
+				if (hb.vars[hb2] != at2) mxThrow("oops");
 
 				if (at1 < at2) std::swap(at1, at2); // outer_at1 unaffected
 				if (hb1 < hb2) std::swap(hb1, hb2); // outer_hb1 unaffected
@@ -376,7 +376,7 @@ ba81ComputeEMFit(omxFitFunction* oo, int want, FitContext *fc)
 					double *iparam2 = omxMatrixColumn(itemParam, item);
 					pda(iparam2, numParam, 1);
 					// Perhaps bounds can be pulled in from librpf? TODO
-					Rf_error("Deriv %d for item %d is %f; are you missing a lbound/ubound?",
+					mxThrow("Deriv %d for item %d is %f; are you missing a lbound/ubound?",
 						 ox, item, deriv0[ox]);
 				}
 
@@ -529,7 +529,7 @@ static void sandwich(omxFitFunction *oo, FitContext *fc)
 	Eigen::ArrayXd &rowWeight = estate->grp.rowMult;
 
 	if (quad.hasBifactorStructure) {
-		Rf_error("Sandwich information matrix method is not implemented for bifactor-optimized models");
+		mxThrow("Sandwich information matrix method is not implemented for bifactor-optimized models");
 	}
 
 	ba81sandwichOp op(numThreads, estate, numParam, state, itemParam, abScale);
@@ -907,7 +907,7 @@ void BA81FitState::compute(int want, FitContext *fc)
 			}
 		}
 	} else {
-		Rf_error("%s: Predict nothing or scores before computing %d", oo->name(), want);
+		mxThrow("%s: Predict nothing or scores before computing %d", oo->name(), want);
 	}
 }
 
@@ -946,7 +946,7 @@ void BA81FitState::init()
 		const double *spec = estate->itemSpec(ix);
 		int id = spec[RPF_ISpecID];
 		if (id < 0 || id >= Glibrpf_numModels) {
-			Rf_error("ItemSpec %d has unknown item model %d", ix, id);
+			mxThrow("ItemSpec %d has unknown item model %d", ix, id);
 		}
 	}
 
